/*
 * templates.cpp - set of utils to make editing templates easier
 * Copyright (C) 2005  Maciej Niedzielski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "templates.h"

#include <qtextedit.h>
#include <qmap.h>
#include <qregexp.h>

//----------------------------------------------------------------------------
// TemplatePopup
//----------------------------------------------------------------------------

class TemplatePopup::Private {
public:
	QTextEdit *edit;
	QMap<int, QString> tags;
};

TemplatePopup::TemplatePopup(QTextEdit *edt, QWidget *parent, const char *name)
	: QPopupMenu(parent, name)
{
	d = new Private;
	d->edit = edt;
	connect(this, SIGNAL(activated(int)), this, SLOT(onActivated(int)));	
}

TemplatePopup::~TemplatePopup()
{
	delete d;
	d = 0;
}

int TemplatePopup::insertTemplateItem(const QString &name, const QString &tag, const QString &/*desc*/)
{
	int id = insertItem(name);
	//setWhatsThis(id, "<b><i>" + tag + "</i></b><br>" + ((desc!="")?desc:name));
	d->tags[id] = tag;

	return id;
}

void TemplatePopup::onActivated(int id)
{
    if (d->edit) {
		d->edit->insert(d->tags[id]);
		d->edit->setFocus();
	}
}       

//----------------------------------------------------------------------------
// TemplateEditHighlighter
//----------------------------------------------------------------------------

int TemplateEditHighlighter::highlightParagraph(const QString & text, int /*endStateOfLastPara*/)
{
	setFormat(0, text.length(), textEdit()->colorGroup().text());

	for ( unsigned i = 0; i < tags.size(); ++i ) {
		int first = 0;
		int start = -1;
		int len = tags[i].length();
		while ( (start = text.find(tags[i], first, false)) != -1 ) {
			setFormat(start, len, textEdit()->colorGroup().link());
				// color of a link - it suggests something 'special' ;-)
				// (and has good contrast)
				
			first = start + len + 1;
		}
	}
	return 0;
}

//----------------------------------------------------------------------------
// expandTemplate function
//----------------------------------------------------------------------------

QString expandTemplate(const QString &t, const ExpandTemplateData &values)
{

	QString res = t;

	QRegExp repl;
	repl.setCaseSensitive(FALSE);

	QString ins;          

	for ( ExpandTemplateData::ConstIterator it = values.begin(); it != values.end(); ++it ) {

		ins = it.data();
		repl.setPattern("<" + it.key() + ">");
		res.replace(repl, ins);

		// every tag has <!tag> version, which adds new line before the value

		if( !ins.isEmpty() ) {
			repl.setPattern("<!" + it.key() + ">");
			res.replace(repl, QString("<br>") + ins);
		}
	}

	// "<>" is a safe way to write "<" inside the template

	ins = "<";
	repl.setPattern("<>");
	res.replace(repl, ins);

	return res;
}

#include "moc_templates.cpp"
